﻿-- Engines
local _, ns = ...
local oUF = ns.oUF or oUF
local S, C, L, DB = unpack(select(2, ...))

-- Variables
local HPTag, PPTag, NameTag, Portrait = nil

-- Begin
local function PortraitFadeIn(...)
    UIFrameFadeOut(HPTag, 0.5, 1, 0)
    UIFrameFadeOut(PPTag, 0.5, 1, 0)
    UIFrameFadeOut(NameTag, 0.5, 1, 0)
    UIFrameFadeIn(Portrait, 0.5, 0, 0.3)
end

local function PortraitFadeOut(...)
    UIFrameFadeIn(HPTag, 0.5, 0, 1)
    UIFrameFadeIn(PPTag, 0.5, 0, 1)
    UIFrameFadeIn(NameTag, 0.5, 0, 1)
    UIFrameFadeOut(Portrait, 0.5, 0.3, 0)
end

local function SetPower(self, ...)
    local power = CreateFrame("StatusBar", nil, self)
    power:SetPoint("BOTTOM", self)
    power:SetSize(self:GetWidth(), 4)
    power:SetStatusBarTexture(DB.Statusbar)
    
    power.bg = power:CreateTexture(nil, "BACKGROUND")
    power.bg:SetTexture(DB.Statusbar)
    power.bg:SetAllPoints()
    power.bg:SetVertexColor(0.12, 0.12, 0.12)
    power.bg.multiplier = 0.2
    
    power.Smooth = true
    power.colorPower = true
    power.frequentUpdates = true
    power.shadow = S.MakeShadow(power, 2)
    
    self.Power = power
end

local function SetHealth(self, ...)
    local health = CreateFrame("StatusBar", nil, self)
    health:SetPoint("TOP", self)
    health:SetStatusBarTexture(DB.Statusbar)
    health:SetSize(self:GetWidth(), self:GetHeight() - 8)
    
    health.bg = health:CreateTexture(nil, "BACKGROUND")
    health.bg:SetAllPoints()
    health.bg:SetTexture(DB.Statusbar)
    health.bg:SetVertexColor(0.12, 0.12, 0.12)
    health.bg.multiplier = 0.12
    
    health.Smooth = true
    health.colorTapping = true
    health.colorDisconnected = true
    health.colorClass = true
    health.colorReaction = true
    health.colorHealth = true
    health.frequentUpdates = true
    health.shadow = S.MakeShadow(health, 2)
    
    self.Health = health
end

local function SetTag(self, ...)
    NameTag = S.MakeText(self.Health, 10)
    NameTag:SetAlpha(0.00)
    NameTag:SetPoint("LEFT", 4, 0)
    self:Tag(NameTag, "[Sora:Level][Sora:Rare][Sora:Color][Sora:Name]|r")
    
    HPTag = S.MakeText(self.Health, 10)
    HPTag:SetAlpha(0.00)
    HPTag:SetPoint("RIGHT", -4, 0)
    self:Tag(HPTag, "[Sora:Color][Sora:Health]|r | [Sora:Color][Sora:PerHealth]|r")
    
    PPTag = S.MakeText(self.Power, 8)
    PPTag:SetAlpha(0.00)
    PPTag:SetPoint("RIGHT", -4, 0)
    self:Tag(PPTag, "[Sora:Power] | [Sora:PerPower]")
end

local function SetAura(self, ...)
    local spacing = 4
    local size = (self:GetWidth() - 4 * 8) / 9
    
    local auras = CreateFrame("Frame", nil, self)
    auras:SetSize(self:GetWidth(), size * 3 + spacing * 2)
    auras:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -spacing)
    
    auras.num = 27
    auras.numBuffs = 9
    auras.gap = true
    auras.size = size
    auras.spacing = spacing
    auras["growth-y"] = "DOWN"
    auras["growth-x"] = "RIGHT"
    auras.onlyShowPlayer = false
    auras.disableCooldown = false
    auras.initialAnchor = "TOPLEFT"
    
    auras.PostCreateIcon = function(self, button)
        if not button.shadow then
            button.shadow = S.MakeShadow(button, 2)
            
            button.icon:SetAllPoints()
            button.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
            
            button.count = S.MakeText(button, 10)
            button.count:SetPoint("BOTTOMRIGHT", 3, 0)
        end
    end
    
    self.Auras = auras
end

local function SetCastbar(self, ...)
    local height = C.UnitFrame.FocusTarget.Height - 4
    local width = self:GetWidth() - C.UnitFrame.FocusTarget.Width - 8 - height;
    
    local bar = CreateFrame("StatusBar", nil, self)
    bar:SetSize(width, height)
    bar:SetStatusBarTexture(DB.Statusbar)
    bar:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -height - 4, 8)

    bar.Shadow = S.MakeShadow(bar, 2)
    bar.BG = bar:CreateTexture(nil, "BACKGROUND")
    bar.BG:SetTexture(DB.Statusbar)
    bar.BG:SetAllPoints()
    bar.BG:SetVertexColor(0.12, 0.12, 0.12, 0.75)
    
    bar.Text = S.MakeText(bar, 10)
    bar.Text:SetPoint("LEFT", 2, 0)
    
    bar.Time = S.MakeText(bar, 10)
    bar.Time:SetPoint("RIGHT", -2, 0)
    
    bar.Icon = bar:CreateTexture(nil, "ARTWORK")
    bar.Icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
    bar.Icon:SetSize(bar:GetHeight(), bar:GetHeight())
    bar.Icon:SetPoint("LEFT", bar, "RIGHT", 4, 0)
    bar.Icon.Shadow = S.MakeTextureShadow(bar, bar.Icon, 2)
    
    bar.SafeZone = bar:CreateTexture(nil, "BORDER")
    bar.SafeZone:SetAlpha(0.25)
    
    bar.CustomTimeText = function(self, duration)
        self.Time:SetText(("%.1f / %.1f"):format(duration, self.max))
    end
    
    bar.CustomDelayText = function(self, duration)
        self.Time:SetText(("%.1f / %.1f"):format(duration, self.max))
    end
    
    bar.PostCastStart = function(self, unit, name, castid, spellid)
        self:SetAlpha(1.00)
        
        if self.notInterruptible then
            self:SetStatusBarColor(1.00, 0.50, 0.50)
        else
            self:SetStatusBarColor(0.37, 0.71, 1.00)
        end
    end
    
    bar.PostChannelStart = function(self, unit, name, castid, spellid)
        self:SetAlpha(1.00)
        
        if self.notInterruptible then
            self:SetStatusBarColor(1.00, 0.50, 0.50)
        else
            self:SetStatusBarColor(0.37, 0.71, 1.00)
        end
    end
    
    bar.timeToHold = 0.50
    
    bar.PostCastFailed = function(self, unit, spellname, castid, spellid)
        UIFrameFadeOut(self, 0.50, 1.00, 0.00)
        self:SetStatusBarColor(1.00, 0.05, 0.00)
    end
    
    bar.PostCastInterrupted = function(self, unit, spellname, castid, spellid)
        UIFrameFadeOut(self, 0.50, 1.00, 0.00)
        self:SetStatusBarColor(1.00, 0.05, 0.00)
    end
    
    self.Castbar = bar
end

local function SetPortrait(self, ...)
    Portrait = CreateFrame("PlayerModel", nil, self.Health)
    Portrait:SetAlpha(0.3)
    Portrait:SetAllPoints()
    Portrait:SetFrameLevel(self.Health:GetFrameLevel() + 1)
    
    self.Portrait = Portrait
end

local function SetRaidIcon(self, ...)
    local RaidIcon = self.Health:CreateTexture(nil, "OVERLAY")
    RaidIcon:SetSize(16, 16)
    RaidIcon:SetPoint("CENTER", self.Health, "TOP", 0, 2)
    
    self.RaidIcon = RaidIcon
end

local function SetCombatIcon(self, ...)
    local Leader = self.Health:CreateTexture(nil, "OVERLAY")
    Leader:SetSize(16, 16)
    Leader:SetPoint("TOPLEFT", self, -7, 9)
    self.Leader = Leader
    
    local Assistant = self.Health:CreateTexture(nil, "OVERLAY")
    Assistant:SetAllPoints(Leader)
    self.Assistant = Assistant
    
    local MasterLooter = self.Health:CreateTexture(nil, "OVERLAY")
    MasterLooter:SetSize(16, 16)
    MasterLooter:SetPoint("LEFT", Leader, "RIGHT")
    self.MasterLooter = MasterLooter
end

local function RegisterForEvent(self, ...)
    self:SetScript("OnLeave", function(self, event, ...)
        UnitFrame_OnLeave(self)

        if not UnitAffectingCombat("player") then
            PortraitFadeIn()
        end
    end)
    
    self:SetScript("OnEnter", function(self, event, ...)
        UnitFrame_OnEnter(self)

        if not UnitAffectingCombat("player") then
            PortraitFadeOut()
        end
    end)
end

local function RegisterStyle(self, ...)
    self:RegisterForClicks("AnyUp")
    self:SetPoint(unpack(C.UnitFrame.Focus.Postion))
    self:SetSize(C.UnitFrame.Focus.Width, C.UnitFrame.Focus.Height)
    
    SetPower(self, ...)
    SetHealth(self, ...)
    
    SetTag(self, ...)
    SetAura(self, ...)
    SetCastbar(self, ...)
    SetPortrait(self, ...)
    SetRaidIcon(self, ...)
    SetCombatIcon(self, ...)
    
    RegisterForEvent(self, ...)
end

local function OnPlayerLogin(self, event, ...)
    oUF:RegisterStyle("oUF_Sora_Focus", RegisterStyle)
    oUF:SetActiveStyle("oUF_Sora_Focus")
    
    local oUFFrame = oUF:Spawn("focus", "oUF_Sora_Focus")
end

local function OnPlayerRegenEnable(self, event, ...)
    PortraitFadeIn()
end

local function OnPlayerRegenDisable(self, event, ...)
    PortraitFadeOut()
end

local Event = CreateFrame("Frame", nil, UIParent)
Event:RegisterEvent("PLAYER_LOGIN")
Event:RegisterEvent("PLAYER_REGEN_ENABLED")
Event:RegisterEvent("PLAYER_REGEN_DISABLED")
Event:SetScript("OnEvent", function(self, event, ...)
    if event == "PLAYER_LOGIN" then
        OnPlayerLogin(self, event, ...)
    elseif event == "PLAYER_REGEN_ENABLED" then
        OnPlayerRegenEnable(self, event, ...)
    elseif event == "PLAYER_REGEN_DISABLED" then
        OnPlayerRegenDisable(self, event, ...)
    end
end)
