-- Engines
local _, ns = ...
local oUF = ns.oUF or oUF
local S, C, L, DB = unpack(select(2, ...))

-- Variables
local HPTag, PPTag, NameTag, Portrait = nil

-- Begin
local function PortraitFadeIn(...)
    UIFrameFadeOut(HPTag, 0.5, 1, 0)
    UIFrameFadeOut(PPTag, 0.5, 1, 0)
    UIFrameFadeOut(NameTag, 0.5, 1, 0)
    UIFrameFadeIn(Portrait, 0.5, 0, 0.3)
end

local function PortraitFadeOut(...)
    UIFrameFadeIn(HPTag, 0.5, 0, 1)
    UIFrameFadeIn(PPTag, 0.5, 0, 1)
    UIFrameFadeIn(NameTag, 0.5, 0, 1)
    UIFrameFadeOut(Portrait, 0.5, 0.3, 0)
end

local function SetPower(self, ...)
    local power = CreateFrame("StatusBar", nil, self)
    power:SetPoint("BOTTOM")
    power:SetSize(self:GetWidth(), 4)
    power:SetStatusBarTexture(DB.Statusbar)
    
    power.bg = power:CreateTexture(nil, "BACKGROUND")
    power.bg:SetTexture(DB.Statusbar)
    power.bg:SetAllPoints()
    power.bg:SetVertexColor(0.12, 0.12, 0.12)
    power.bg.multiplier = 0.12
    
    power.Smooth = true
    power.colorPower = true
    power.frequentUpdates = true
    power.shadow = S.MakeShadow(power, 2)
    
    self.Power = power
end

local function SetHealth(self, ...)
    local health = CreateFrame("StatusBar", nil, self)
    health:SetPoint("TOP")
    health:SetStatusBarTexture(DB.Statusbar)
    health:SetSize(self:GetWidth(), self:GetHeight() - 8)
    
    health.bg = health:CreateTexture(nil, "BACKGROUND")
    health.bg:SetAllPoints()
    health.bg:SetTexture(DB.Statusbar)
    health.bg:SetVertexColor(0.12, 0.12, 0.12)
    health.bg.multiplier = 0.12
    
    health.Smooth = true
    health.colorTapping = true
    health.colorDisconnected = true
    health.colorClass = true
    health.colorReaction = true
    health.colorHealth = true
    health.frequentUpdates = true
    health.shadow = S.MakeShadow(health, 2)
    
    self.Health = health
end

local function SetTag(self, ...)
    NameTag = S.MakeText(self.Health, 10)
    NameTag:SetAlpha(0.00)
    NameTag:SetPoint("LEFT", 4, 0)
    self:Tag(NameTag, "[Sora:Level][Sora:Rare][Sora:Color][Sora:Name]|r")
    
    HPTag = S.MakeText(self.Health, 10)
    HPTag:SetAlpha(0.00)
    HPTag:SetPoint("RIGHT", -4, 0)
    self:Tag(HPTag, "[Sora:Color][Sora:Health]|r | [Sora:Color][Sora:PerHealth]|r")
    
    PPTag = S.MakeText(self.Power, 8)
    PPTag:SetAlpha(0.00)
    PPTag:SetPoint("RIGHT", -4, 0)
    self:Tag(PPTag, "[Sora:Power] | [Sora:PerPower]")
end

local function SetDebuff(self, ...)
    local spacing = 4
    local size = (self:GetWidth() - 4 * 8) / 9
    
    local debuffs = CreateFrame("Frame", nil, self)
    debuffs:SetSize(self:GetWidth(), size * 2 + spacing)
    debuffs:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -spacing)
    
    debuffs.num = 18
    debuffs.size = size
    debuffs.spacing = spacing
    debuffs["growth-y"] = "DOWN"
    debuffs["growth-x"] = "RIGHT"
    debuffs.disableCooldown = false
    debuffs.initialAnchor = "TOPLEFT"
    
    debuffs.PostCreateIcon = function(self, icon, ...)
        if not icon._isProcessed then
            icon.shadow = S.MakeShadow(icon, 2)
            
            icon.icon:SetAllPoints()
            icon.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
            
            icon.count = S.MakeText(icon, 10)
            icon.count:SetPoint("BOTTOMRIGHT", 3, 0)
            
            icon._isProcessed = true
        end
    end
    
    self.Debuffs = debuffs
end

local function SetCastbar(self, ...)
    local bar = CreateFrame("StatusBar", nil, self)
    bar:SetStatusBarTexture(DB.Statusbar)
    bar:SetSize(C.ActionBar.Size * 18 + C.ActionBar.Space * 17 - C.ActionBar.Space - 32, 32)
    bar:SetPoint("BOTTOMLEFT", "MultiBarBottomRightButton1", "TOPLEFT", 0, C.ActionBar.Size + C.ActionBar.Space + 8)
    
    bar.Shadow = S.MakeShadow(bar, 2)
    bar.BG = bar:CreateTexture(nil, "BACKGROUND")
    bar.BG:SetTexture(DB.Statusbar)
    bar.BG:SetAllPoints()
    bar.BG:SetVertexColor(0.12, 0.12, 0.12, 0.75)
    
    bar.Text = S.MakeText(bar, 10)
    bar.Text:SetPoint("LEFT", 2, 0)
    
    bar.Time = S.MakeText(bar, 10)
    bar.Time:SetPoint("RIGHT", -2, 0)
    
    bar.Icon = bar:CreateTexture(nil, "ARTWORK")
    bar.Icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
    bar.Icon:SetSize(bar:GetHeight(), bar:GetHeight())
    bar.Icon:SetPoint("LEFT", bar, "RIGHT", C.ActionBar.Space, 0)
    bar.Icon.Shadow = S.MakeTextureShadow(bar, bar.Icon, 2)
    
    bar.SafeZone = bar:CreateTexture(nil, "BORDER")
    bar.SafeZone:SetAlpha(0.25)
    
    bar.CustomTimeText = function(self, duration)
        self.Time:SetText(("%.1f / %.1f"):format(duration, self.max))
    end
    
    bar.CustomDelayText = function(self, duration)
        self.Time:SetText(("%.1f / %.1f"):format(duration, self.max))
    end
    
    bar.PostCastStart = function(self, unit, name, castid, spellid)
        self:SetAlpha(1.00)
        
        if self.notInterruptible then
            self:SetStatusBarColor(1.00, 0.50, 0.50)
        else
            self:SetStatusBarColor(0.37, 0.71, 1.00)
        end
    end
    
    bar.PostChannelStart = function(self, unit, name, castid, spellid)
        self:SetAlpha(1.00)
        
        if self.notInterruptible then
            self:SetStatusBarColor(1.00, 0.50, 0.50)
        else
            self:SetStatusBarColor(0.37, 0.71, 1.00)
        end
    end
    
    bar.timeToHold = 0.50
    
    bar.PostCastFailed = function(self, unit, spellname, castid, spellid)
        UIFrameFadeOut(self, 0.50, 1.00, 0.00)
        self:SetStatusBarColor(1.00, 0.05, 0.00)
    end
    
    bar.PostCastInterrupted = function(self, unit, spellname, castid, spellid)
        UIFrameFadeOut(self, 0.50, 1.00, 0.00)
        self:SetStatusBarColor(1.00, 0.05, 0.00)
    end
    
    self.Castbar = bar
end

local function SetPortrait(self, ...)
    Portrait = CreateFrame("PlayerModel", nil, self.Health)
    Portrait:SetAlpha(0.3)
    Portrait:SetAllPoints()
    Portrait:SetFrameLevel(self.Health:GetFrameLevel() + 1)
    
    self.Portrait = Portrait
end

local function SetRaidIcon(self, ...)
    local RaidIcon = self.Health:CreateTexture(nil, "OVERLAY")
    RaidIcon:SetSize(16, 16)
    RaidIcon:SetPoint("CENTER", self.Health, "TOP", 0, 0)
    
    self.RaidIcon = RaidIcon
end

local function SetMiscIcon(self, ...)
    local Leader = self.Health:CreateTexture(nil, "OVERLAY")
    Leader:SetSize(16, 16)
    Leader:SetPoint("TOPLEFT", self, -7, 9)
    self.Leader = Leader
    
    local Assistant = self.Health:CreateTexture(nil, "OVERLAY")
    Assistant:SetAllPoints(Leader)
    self.Assistant = Assistant
    
    local MasterLooter = self.Health:CreateTexture(nil, "OVERLAY")
    MasterLooter:SetSize(16, 16)
    MasterLooter:SetPoint("LEFT", Leader, "RIGHT")
    self.MasterLooter = MasterLooter
    
    local Resting = self.Health:CreateTexture(nil, "OVERLAY")
    Resting:SetSize(16, 16)
    Resting:SetPoint("TOPRIGHT", self.Health, "TOPLEFT", 0, 4)
    self.Resting = Resting
    
    local Combat = self.Health:CreateTexture(nil, "OVERLAY")
    Combat:SetSize(16, 16)
    Combat:SetPoint("BOTTOMRIGHT", self.Health, "BOTTOMLEFT", 0, -4)
    self.Combat = Combat
end

local function SetClassIcons(self, ...)
    if DB.MyClass == "DEATHKNIGHT" then
        local Runes = {}
        
        for i = 1, 6 do
            local Rune = CreateFrame("StatusBar", nil, self)
            Rune:SetStatusBarTexture(DB.Statusbar)
            Rune:SetSize((self:GetWidth() - 20) / 6, 4)
            Rune:SetStatusBarColor(0.50, 0.33, 0.67)
            
            Rune.BG = Rune:CreateTexture(nil, "BACKGROUND")
            Rune.BG:SetAllPoints()
            Rune.BG:SetTexture(DB.Statusbar)
            Rune.BG:SetVertexColor(0.12, 0.12, 0.12)
            Rune.Shadow = S.MakeShadow(Rune, 2)
            
            if i == 1 then
                Rune:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
            else
                Rune:SetPoint("LEFT", Runes[i - 1], "RIGHT", 4, 0)
            end
            
            Runes[i] = Rune
        end
        
        self.Runes = Runes
    else
        local Colors = {}
        Colors["MAGE"] = {0.83, 0.50, 0.83}
        Colors["MONK"] = {0.00, 0.80, 0.60}
        Colors["PALADIN"] = {1.00, 1.00, 0.40}
        Colors["WARLOCK"] = {0.67, 0.33, 0.67}
        
        local ClassIcons = {}
        
        for i = 1, 8 do
            local ClassIcon = CreateFrame("StatusBar", nil, self)
            ClassIcon:SetStatusBarTexture(DB.Statusbar)
            ClassIcon:SetStatusBarColor(unpack(Colors[DB.MyClass] or {0.81, 0.03, 0.03}))
            
            ClassIcon.BG = CreateFrame("StatusBar", nil, self)
            ClassIcon.BG:SetFrameLevel(0)
            ClassIcon.BG:SetStatusBarTexture(DB.Statusbar)
            ClassIcon.BG:SetStatusBarColor(0.12, 0.12, 0.12)
            ClassIcon.BG.Shadow = S.MakeShadow(ClassIcon.BG, 2)
            
            ClassIcons[i] = ClassIcon
        end
        
        ClassIcons.PostUpdate = function(element, cur, max, hasMaxChanged, powerType, event)
            if not max then
                return
            end
            
            if hasMaxChanged then
                for i = max, 8 do
                    ClassIcons[i]:ClearAllPoints()
                    ClassIcons[i].BG:ClearAllPoints()
                end
                
                for i = 1, max do
                    ClassIcons[i]:SetSize((self:GetWidth() - (max - 1) * 4) / max, 4)
                    ClassIcons[i].BG:SetSize((self:GetWidth() - (max - 1) * 4) / max, 4)
                    
                    if i == 1 then
                        ClassIcons[i]:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
                        ClassIcons[i].BG:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
                    else
                        ClassIcons[i]:SetPoint("LEFT", ClassIcons[i - 1], "RIGHT", 4, 0)
                        ClassIcons[i].BG:SetPoint("LEFT", ClassIcons[i - 1], "RIGHT", 4, 0)
                    end
                end
            end
        end
        
        self.ClassIcons = ClassIcons
    end
end

local function RegisterForEvent(self, ...)
    self:SetScript("OnLeave", function(self, event, ...)
        UnitFrame_OnLeave(self)
        
        if not UnitAffectingCombat("player") then
            PortraitFadeIn()
        end
    end)
    
    self:SetScript("OnEnter", function(self, event, ...)
        UnitFrame_OnEnter(self)
        
        if not UnitAffectingCombat("player") then
            PortraitFadeOut()
        end
    end)
end

local function RegisterStyle(self, ...)
    self:RegisterForClicks("AnyDown")
    self:SetPoint(unpack(C.UnitFrame.Player.Postion))
    self:SetSize(C.UnitFrame.Player.Width, C.UnitFrame.Player.Height)
    
    SetPower(self, ...)
    SetHealth(self, ...)
    
    SetTag(self, ...)
    SetDebuff(self, ...)
    SetCastbar(self, ...)
    SetPortrait(self, ...)
    SetRaidIcon(self, ...)
    SetMiscIcon(self, ...)
    SetClassIcons(self, ...)
    
    RegisterForEvent(self, ...)
end

local function OnPlayerLogin(self, event, ...)
    oUF:RegisterStyle("oUF_Sora_Player", RegisterStyle)
    oUF:SetActiveStyle("oUF_Sora_Player")
    
    local oUFFrame = oUF:Spawn("player", "oUF_Sora_Player")
end

local function OnPlayerRegenEnable(self, event, ...)
    PortraitFadeIn()
end

local function OnPlayerRegenDisable(self, event, ...)
    PortraitFadeOut()
end

local Event = CreateFrame("Frame", nil, UIParent)
Event:RegisterEvent("PLAYER_LOGIN")
Event:RegisterEvent("PLAYER_REGEN_ENABLED")
Event:RegisterEvent("PLAYER_REGEN_DISABLED")
Event:SetScript("OnEvent", function(self, event, ...)
    if event == "PLAYER_LOGIN" then
        OnPlayerLogin(self, event, ...)
    elseif event == "PLAYER_REGEN_ENABLED" then
        OnPlayerRegenEnable(self, event, ...)
    elseif event == "PLAYER_REGEN_DISABLED" then
        OnPlayerRegenDisable(self, event, ...)
    end
end)
